#include <iostream>
#include <cstdio>

#include <gtkmm/plug.h>
#include <gtkmm/window.h>
#include <gdkmm.h>
#include <glibmm.h>

#include <vte/vte.h>

#include "toscoterm.h"


static void set_window_geometry_hints(Gtk::Window *window, VteTerminal *term)
{
	Gdk::Geometry hints;
	gint xpad;
	gint ypad;

	vte_terminal_get_padding(VTE_TERMINAL(term), &xpad, &ypad);

	hints.base_width = xpad;
	hints.base_height = ypad;
	hints.width_inc = VTE_TERMINAL(term)->char_width;
	hints.height_inc = VTE_TERMINAL(term)->char_height;
	hints.min_width = hints.base_width + hints.width_inc * 4;
	hints.min_height = hints.base_height + hints.height_inc * 2;

	window->set_geometry_hints(*Glib::wrap(GTK_WIDGET(term)), hints, Gdk::HINT_RESIZE_INC | Gdk::HINT_MIN_SIZE | Gdk::HINT_BASE_SIZE);
}

static void childexited_cb(VteTerminal *term)
{
	gtk_exit(0);
}

toscoterm::toscoterm(int argc, char **argv)
{
	unsigned xid = 0;
	int fullscreen = 0;
	char *exec = NULL;
	char *termname = NULL;

	for (int i = 0; i < argc; i++) {
		if (!strncmp(argv[i], "-x", 2))
			xid = strtol(argv[++i], 0, 0);
	}

	if (xid) {
		window = new Gtk::Plug(xid);
		printf("X11 ID %u\n", (dynamic_cast<Gtk::Plug *>(window))->get_id());
	} else {
		window = new Gtk::Window(Gtk::WINDOW_TOPLEVEL);
		window->set_title("Terminal");
	}
	window->set_default_size(640, 480);

	GtkWidget *term = vte_terminal_new();
	set_window_geometry_hints(window, VTE_TERMINAL(term));

	vte_terminal_set_emulation(VTE_TERMINAL(term), termname ? termname : "xterm");
	vte_terminal_set_size(VTE_TERMINAL(term), 80, 24);

	if (!exec)
		vte_terminal_fork_command(VTE_TERMINAL(term), g_getenv("SHELL"), NULL, NULL, ".", FALSE, FALSE, FALSE);
	else {
		gchar **command;
		g_shell_parse_argv(exec, NULL, &command, NULL);
		vte_terminal_fork_command(VTE_TERMINAL(term), (const char *)*(command), command, NULL, ".", FALSE, FALSE, FALSE);
	}

	window->add(*Glib::wrap(GTK_WIDGET(term)));
	window->set_focus(*Glib::wrap(GTK_WIDGET(term)));
//	window->grab_focus();
	window->show_all_children();


	g_signal_connect(term, "child-exited", G_CALLBACK(childexited_cb), NULL);
}


toscoterm::~toscoterm()
{
	delete window;
}
